==> oneDPL-2021.7.1.tar.gz: oneDPL-oneDPL-2021.7.1-release/test/xpu_api/cmath/nearbyint/xpu_nearbyint.pass.cpp <==
// -*- C++ -*-
//===----------------------------------------------------------------------===//
//
// Copyright (C) Intel Corporation
//
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
// This file incorporates work covered by the following copyright and permission
// notice:
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
//
//===----------------------------------------------------------------------===//

#include <oneapi/dpl/cmath>

#include "support/test_config.h"
#include "support/utils.h"
#include "support/utils_invoke.h"

#include <iostream>
#include <vector>

#if TEST_DPCPP_BACKEND_PRESENT
#include <CL/sycl.hpp>

constexpr cl::sycl::access::mode sycl_read = cl::sycl::access::mode::read;
constexpr cl::sycl::access::mode sycl_write = cl::sycl::access::mode::write;

template <typename KernelClass, typename Function, typename ValueType>
void
test(Function fnc, const std::vector<ValueType>& args, const char* message)
{
    const auto args_count = args.size();

    std::vector<ValueType> output(args_count);

    cl::sycl::range<1> numOfItems1{args_count};
    cl::sycl::range<1> numOfItems2{args_count};

    // Evaluate results in Kernel
    {
        cl::sycl::queue deviceQueue(TestUtils::async_handler);

        if (!TestUtils::has_type_support<ValueType>(deviceQueue.get_device()))
        {
            std::cout << deviceQueue.get_device().template get_info<sycl::info::device::name>()
                      << " does not support " << typeid(ValueType).name() << " type,"
                      << " affected test case have been skipped" << std::endl;
            return;
        }

        cl::sycl::buffer<ValueType> buffer1(args.data(), args_count);
        cl::sycl::buffer<ValueType> buffer2(output.data(), args_count);

        deviceQueue.submit(
            [&](cl::sycl::handler& cgh)
            {
                auto in = buffer1.template get_access<sycl_read>(cgh);
                auto out = buffer2.template get_access<sycl_write>(cgh);
                cgh.single_task<KernelClass>(
                    [=]()
                    {
                        for (size_t i = 0; i < args_count; ++i)
                            out[i] = fnc(in[i]);
                    });
            });
    }

    // Check results: compare resuls evaluated in Kernel and on host
    for (size_t i = 0; i < args_count; ++i)
    {
        auto host_result = fnc(args[i]);
        EXPECT_EQ(host_result, output[i], message);
    }
}

class Test;

#endif // TEST_DPCPP_BACKEND_PRESENT

int
main()
{
#if TEST_DPCPP_BACKEND_PRESENT

    // functions from https://en.cppreference.com/w/cpp/numeric/math/nearbyint :
    //     float  nearbyint (float arg);
    //     float  nearbyintf(float arg);
    //     double nearbyint (double arg);

    ////////////////////////////////////////////////////////
    // float nearbyint(float arg);
    auto f_nearbyint_float = [](float arg) -> float { return oneapi::dpl::nearbyint(arg); };
    const std::vector<float> f_args_float = {+2.3, +2.5, +3.5, -2.3, -2.5, -3.5};
    test<TestUtils::unique_kernel_name<Test, 1>>(f_nearbyint_float, f_args_float, "float nearbyint(float)");

    ////////////////////////////////////////////////////////
    // float nearbyintf(float arg);
    auto f_nearbyintf_float = [](float arg) -> float { return oneapi::dpl::nearbyintf(arg); };
    test<TestUtils::unique_kernel_name<Test, 11>>(f_nearbyintf_float, f_args_float, "float nearbyintf(float)");

    ////////////////////////////////////////////////////////
    // double nearbyint(double arg);
    auto f_nearbyint_double = [](double arg) -> double { return oneapi::dpl::nearbyint(arg); };
    const std::vector<double> f_args_double = {+2.3, +2.5, +3.5, -2.3, -2.5, -3.5};
    test<TestUtils::unique_kernel_name<Test, 2>>(f_nearbyint_double, f_args_double, "double nearbyint(double)");

#endif // TEST_DPCPP_BACKEND_PRESENT

    return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT);
}

==> oneDPL-2022.0.0.tar.gz: oneDPL-oneDPL-2022.0.0-release/test/xpu_api/cmath/nearbyint/xpu_nearbyint.pass.cpp <==
// -*- C++ -*-
//===----------------------------------------------------------------------===//
//
// Copyright (C) Intel Corporation
//
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
// This file incorporates work covered by the following copyright and permission
// notice:
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
//
//===----------------------------------------------------------------------===//

#include <oneapi/dpl/cmath>

#include "support/test_config.h"
#include "support/utils.h"
#include "support/utils_invoke.h"

#include <iostream>
#include <vector>

#if TEST_DPCPP_BACKEND_PRESENT
#include <CL/sycl.hpp>

constexpr sycl::access::mode sycl_read = sycl::access::mode::read;
constexpr sycl::access::mode sycl_write = sycl::access::mode::write;

template <typename KernelClass, typename Function, typename ValueType>
void
test(Function fnc, const std::vector<ValueType>& args, const char* message)
{
    const auto args_count = args.size();

    std::vector<ValueType> output(args_count);

    sycl::range<1> numOfItems1{args_count};
    sycl::range<1> numOfItems2{args_count};

    // Evaluate results in Kernel
    {
        sycl::queue deviceQueue(TestUtils::async_handler);

        if (!TestUtils::has_type_support<ValueType>(deviceQueue.get_device()))
        {
            std::cout << deviceQueue.get_device().template get_info<sycl::info::device::name>()
                      << " does not support " << typeid(ValueType).name() << " type,"
                      << " affected test case have been skipped" << std::endl;
            return;
        }

        sycl::buffer<ValueType> buffer1(args.data(), args_count);
        sycl::buffer<ValueType> buffer2(output.data(), args_count);

        deviceQueue.submit(
            [&](sycl::handler& cgh)
            {
                auto in = buffer1.template get_access<sycl_read>(cgh);
                auto out = buffer2.template get_access<sycl_write>(cgh);
                cgh.single_task<KernelClass>(
                    [=]()
                    {
                        for (size_t i = 0; i < args_count; ++i)
                            out[i] = fnc(in[i]);
                    });
            });
    }

    // Check results: compare resuls evaluated in Kernel and on host
    for (size_t i = 0; i < args_count; ++i)
    {
        auto host_result = fnc(args[i]);
        EXPECT_EQ(host_result, output[i], message);
    }
}

class Test;

#endif // TEST_DPCPP_BACKEND_PRESENT

int
main()
{
#if TEST_DPCPP_BACKEND_PRESENT

    // functions from https://en.cppreference.com/w/cpp/numeric/math/nearbyint :
    //     float  nearbyint (float arg);
    //     float  nearbyintf(float arg);
    //     double nearbyint (double arg);

    ////////////////////////////////////////////////////////
    // float nearbyint(float arg);
    auto f_nearbyint_float = [](float arg) -> float { return oneapi::dpl::nearbyint(arg); };
    const std::vector<float> f_args_float = {+2.3, +2.5, +3.5, -2.3, -2.5, -3.5};
    test<TestUtils::unique_kernel_name<Test, 1>>(f_nearbyint_float, f_args_float, "float nearbyint(float)");

    ////////////////////////////////////////////////////////
    // float nearbyintf(float arg);
    auto f_nearbyintf_float = [](float arg) -> float { return oneapi::dpl::nearbyintf(arg); };
    test<TestUtils::unique_kernel_name<Test, 11>>(f_nearbyintf_float, f_args_float, "float nearbyintf(float)");

    ////////////////////////////////////////////////////////
    // double nearbyint(double arg);
    auto f_nearbyint_double = [](double arg) -> double { return oneapi::dpl::nearbyint(arg); };
    const std::vector<double> f_args_double = {+2.3, +2.5, +3.5, -2.3, -2.5, -3.5};
    test<TestUtils::unique_kernel_name<Test, 2>>(f_nearbyint_double, f_args_double, "double nearbyint(double)");

#endif // TEST_DPCPP_BACKEND_PRESENT

    return TestUtils::done(TEST_DPCPP_BACKEND_PRESENT);
}
